package com.ruoyi.project.check.mapper;

import java.util.List;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.ruoyi.common.annotation.DataScopePlant;
import com.ruoyi.project.check.domain.TCheckCheckpoints;
import com.ruoyi.project.check.domain.TCheckRepairpoints;

/**
 * 维修点Mapper接口
 *
 * @author ruoyi
 * @date 2022-12-06
 */
public interface TCheckRepairpointsMapper  extends BaseMapper<TCheckRepairpoints>
{
    /**
     * 查询维修点
     *
     * @param checkId 维修点主键
     * @return 维修点
     */
    public TCheckRepairpoints selectTCheckRepairpointsByCheckId(Long checkId);

    /**
     * 查询维修点列表
     *
     * @param tCheckRepairpoints 维修点
     * @return 维修点集合
     */
    @DataScopePlant(deptAlias = "d")
    public List<TCheckRepairpoints> selectTCheckRepairpointsList(TCheckRepairpoints tCheckRepairpoints);

    public List<TCheckRepairpoints> selectTCheckRepairpointsByIds(Long[] ids);
    public List<TCheckRepairpoints> selectRepairpointsByInspectionId(Long inspectionId);

    /**
     * 新增维修点
     *
     * @param tCheckRepairpoints 维修点
     * @return 结果
     */
    public int insertTCheckRepairpoints(TCheckRepairpoints tCheckRepairpoints);
    public int insertTCheckRepairpointsByList(List<TCheckRepairpoints> tCheckRepairpoints);

    /**
     * 修改维修点
     *
     * @param tCheckRepairpoints 维修点
     * @return 结果
     */
    public int updateTCheckRepairpoints(TCheckRepairpoints tCheckRepairpoints);
    public int updateTCheckRepairpointsByCheckIds(TCheckRepairpoints tCheckRepairpoints);

    /**
     * 删除维修点
     *
     * @param checkId 维修点主键
     * @return 结果
     */
    public int deleteTCheckRepairpointsByCheckId(Long checkId);
    public int updateTCheckRepairpointsByUpload(TCheckRepairpoints tCheckRepairpoints);
    public int updateApproveStatusByIds(TCheckRepairpoints tCheckRepairpoints);

    /**
     * 批量删除维修点
     *
     * @param checkIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteTCheckRepairpointsByCheckIds(Long[] checkIds);
}
